'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.verticalScrollConfig = exports.row = exports.gutters = exports.rowGutter = exports.horizontalScrollConfig = exports.headerType = exports.column = exports.guttersConfig = exports.gutterConfig = exports.renderableComponent = undefined;
exports.list = list;
exports.lists = lists;

var _react = require('react');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _constants = require('./constants');

var constants = _interopRequireWildcard(_constants);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var renderableComponent = exports.renderableComponent = _react.PropTypes.oneOfType([_react.PropTypes.func, _react.PropTypes.element]);

var gutterConfig = exports.gutterConfig = _react.PropTypes.shape({
  className: _react.PropTypes.string,
  handleClassName: _react.PropTypes.string,
  handleWidth: _react.PropTypes.number,
  minPosition: _react.PropTypes.number,
  maxPosition: _react.PropTypes.number,
  position: _react.PropTypes.number.isRequired,
  onResize: _react.PropTypes.func
});

var guttersConfig = exports.guttersConfig = _react.PropTypes.shape({
  left: gutterConfig,
  right: gutterConfig
});

var column = exports.column = _react.PropTypes.oneOf(_lodash2.default.values(constants.columns));

var headerType = exports.headerType = _react.PropTypes.oneOf(_lodash2.default.values(constants.headerType));

var horizontalScrollConfig = exports.horizontalScrollConfig = _react.PropTypes.shape({
  className: _react.PropTypes.string,
  contentWidth: _react.PropTypes.number.isRequired,
  onScroll: _react.PropTypes.function,
  passthroughOffsets: _react.PropTypes.bool,
  scaleWithCenterContent: _react.PropTypes.bool,
  scrollbarHeight: _react.PropTypes.number
});

var rowGutter = exports.rowGutter = _react.PropTypes.shape({
  className: _react.PropTypes.string,
  contentComponent: renderableComponent.isRequired,
  handleClassName: _react.PropTypes.string,
  props: _react.PropTypes.object
});

var gutters = exports.gutters = _react.PropTypes.shape({
  left: rowGutter,
  right: rowGutter
});

var row = exports.row = _react.PropTypes.shape({
  className: _react.PropTypes.string,
  contentClassName: _react.PropTypes.string,
  contentComponent: renderableComponent.isRequired,
  gutters: gutters,
  height: _react.PropTypes.number.isRequired,
  key: _react.PropTypes.string,
  props: _react.PropTypes.object
});

var verticalScrollConfig = exports.verticalScrollConfig = _react.PropTypes.shape({
  className: _react.PropTypes.string,
  onScroll: _react.PropTypes.function,
  scrollbarWidth: _react.PropTypes.number
});

function findInvalid(container, fn) {
  var invalid = null;
  _lodash2.default.forEach(container, function (innerContainer, index) {
    invalid = fn(innerContainer, index);
    if (invalid) {
      return false;
    }
    return true;
  });

  return invalid;
}

function validateRenderable(props, propName, location) {
  if (!_lodash2.default.isFunction(props[propName]) && !(0, _react.isValidElement)(props[propName])) {
    return new Error('Invalid ' + location + ' `' + propName + '` supplied to `Rickscroll`.');
  }
  return null;
}

function validateGutter(props, side, location) {
  var gutterProp = props[side];

  var fullLocation = location + '.side';

  if (_lodash2.default.isUndefined(gutterProp)) {
    return null;
  }

  if (!_lodash2.default.isObject(gutterProp)) {
    return new Error('Invalid ' + location + ' `' + side + '` supplied to `Rickscroll`.');
  }

  if (!_lodash2.default.isUndefined(gutterProp.className) && !_lodash2.default.isString(gutterProp.className)) {
    return new Error('Invalid ' + fullLocation + ' `className` supplied to `Rickscroll`.');
  }

  var invalid = validateRenderable(gutterProp, 'contentComponent', fullLocation);
  if (invalid) {
    return invalid;
  }

  if (!_lodash2.default.isUndefined(gutterProp.handleClassName) && !_lodash2.default.isString(gutterProp.handleClassName)) {
    return new Error('Invalid ' + fullLocation + ' `handleClassName` supplied to `Rickscroll`.');
  }

  if (!_lodash2.default.isUndefined(gutterProp.props) && !_lodash2.default.isObject(gutterProp.props)) {
    return new Error('Invalid prop at ' + location + ' `props` supplied to `Rickscroll`.');
  }

  return null;
}

function validateGutters(props, location) {
  var guttersProp = props.gutters;

  var fullLocation = location + '.gutters';

  if (_lodash2.default.isUndefined(guttersProp)) {
    return null;
  }

  if (!_lodash2.default.isObject(guttersProp)) {
    return new Error('Invalid ' + location + ' `gutters` supplied to `Rickscroll`.');
  }

  var invalid = validateGutter(props, 'left', fullLocation);
  if (invalid) {
    return invalid;
  }

  invalid = validateGutter(props, 'right', fullLocation);
  if (invalid) {
    return invalid;
  }

  return null;
}

function validateRow(props, location) {
  if (!_lodash2.default.isUndefined(props.className) && !_lodash2.default.isString(props.className)) {
    return new Error('Invalid ' + location + ' `className` supplied to `Rickscroll`.');
  }

  if (!_lodash2.default.isUndefined(props.contentClassName) && !_lodash2.default.isString(props.contentClassName)) {
    return new Error('Invalid ' + location + ' `contentClassName` supplied to `Rickscroll`.');
  }

  var invalid = validateRenderable(props, 'contentComponent', location);
  if (invalid) {
    return invalid;
  }

  invalid = validateGutters(props, location);
  if (invalid) {
    return invalid;
  }

  if (!_lodash2.default.isNumber(props.height)) {
    return new Error('Invalid prop at ' + location + ' `height` supplied to `Rickscroll`.');
  }

  if (!_lodash2.default.isUndefined(props.props) && !_lodash2.default.isObject(props.props)) {
    return new Error('Invalid prop at ' + location + ' `props` supplied to `Rickscroll`.');
  }

  return null;
}

function validateRows(rows, locationSuffix) {
  var keySet = arguments.length <= 2 || arguments[2] === undefined ? new Set() : arguments[2];

  var initialKeyCount = keySet.size;
  var invalidRow = findInvalid(rows, function (listRow, index) {
    if (listRow.key === 0 || listRow.key) {
      // covers the case where we have a duplicate key
      if (keySet.has(listRow.key)) {
        return new Error('Invalid props supplied to `Rickscroll`. Duplicate row key ' + listRow.key + '.');
      }

      keySet.add(listRow.key);
    } else if (keySet.size) {
      return new Error('Invalid props supplied to `Rickscroll`. Missing row key at ' + index + '.');
    }

    return validateRow(listRow, locationSuffix + '[' + index + ']');
  });

  if (invalidRow) {
    return invalidRow;
  }

  // covers the case where some rows don't have a key
  if (keySet.size !== 0 && keySet.size - initialKeyCount !== rows.length) {
    return new Error('Invalid props supplied to `Rickscroll`. Missing keys on row declarations.');
  }

  return null;
}

function list(props) {
  var listProp = props.list;
  var listsProp = props.lists;


  if (!listProp && !listsProp || listProp && listsProp) {
    return new Error('Invalid props supplied to `Rickscroll`. Must supply at maximum one of list or lists.');
  }

  if (!listProp && listsProp) {
    return null;
  }

  if (!_lodash2.default.isArray(listProp)) {
    return new Error('Invalid prop `list` supplied to `Rickscroll`. Must be an array.');
  }

  return validateRows(listProp, 'list');
}

function lists(props) {
  var listProp = props.list;
  var listsProp = props.lists;


  if (!listProp && !listsProp || listProp && listsProp) {
    return new Error('Invalid props supplied to `Rickscroll`. Must supply at maximum one of list or lists.');
  }

  if (listProp && !listsProp) {
    return null;
  }

  if (!_lodash2.default.isArray(listsProp)) {
    return new Error('Invalid prop `lists` supplied to `Rickscroll`. Must be an array.');
  }

  // used to catch duplicate or missing keys
  var keySet = new Set();
  var numberOfNonHeaderRows = 0;

  var listsAreInvalid = findInvalid(listsProp, function (listContainer, containerIndex) {
    if (!_lodash2.default.isUndefined(props.headerClassName) && !_lodash2.default.isString(props.headerClassName)) {
      return new Error('Invalid lists[' + containerIndex + '] `headerClassName` supplied to `Rickscroll`.');
    }

    var invalidRenderable = validateRenderable(listContainer, 'headerComponent', 'lists[' + containerIndex + ']');
    if (invalidRenderable) {
      return invalidRenderable;
    }

    if (listContainer.headerKey === 0 || listContainer.headerKey) {
      if (keySet.has(listContainer.headerKey)) {
        return new Error('Duplicate `headerKey` at lists[' + containerIndex + '] supplied to `Rickscroll`.');
      }
      keySet.add(listContainer.headerKey);
    } else if (keySet.size) {
      return new Error('Invalid props supplied to `Rickscroll`. Missing headerKey at lists[' + containerIndex + '].');
    }

    if (!_lodash2.default.isUndefined(listContainer.headerProps) && !_lodash2.default.isObject(listContainer.headerProps)) {
      return new Error('Invalid prop at lists[' + containerIndex + '] `headerProps` supplied to `Rickscroll`.');
    }

    if (!_lodash2.default.isNumber(listContainer.height)) {
      return new Error('Invalid prop at lists[' + containerIndex + '] `height` supplied to `Rickscroll`.');
    }

    if (!_lodash2.default.isUndefined(props.initCollapsed) && !_lodash2.default.isBoolean(props.initCollapsed)) {
      return new Error('Invalid lists[' + containerIndex + '] `initCollapsed` supplied to `Rickscroll`.');
    }

    var initalKeyCount = keySet.size;
    var rowsInvalid = validateRows(listContainer.rows, 'lists.rows', keySet);
    if (rowsInvalid) {
      return rowsInvalid;
    }

    if (listContainer.headerKey && listContainer.rows.length && keySet.size === initalKeyCount) {
      return new Error('Invalid props supplied to `Rickscroll`. headerKey supplied, but no keys defined for ' + ('lists[' + containerIndex + '] `rows`.'));
    }

    if (!listContainer.headerKey && keySet.size !== initalKeyCount) {
      return new Error('Invalid props supplied to `Rickscroll`. No headerKey supplied to lists[' + containerIndex + ']');
    }

    numberOfNonHeaderRows += listContainer.rows.length;

    return null;
  });

  if (listsAreInvalid) {
    return listsAreInvalid;
  }

  if (keySet.size !== 0 && numberOfNonHeaderRows + listsProp.length !== keySet.size) {
    return new Error('Invalid props supplied to `Rickscroll`. Missing keys on header declarations.');
  }

  return null;
}