'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.returnWidthIfComponentExists = exports.getGutterWidths = exports.getResizeValues = exports.triggerAnimationFrameCreator = undefined;
exports.avgRowHeight = avgRowHeight;
exports.buildRowConfig = buildRowConfig;
exports.getMaxHeight = getMaxHeight;
exports.getVerticalScrollValues = getVerticalScrollValues;
exports.getWidthStyle = getWidthStyle;

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _constants = require('./constants');

var constants = _interopRequireWildcard(_constants);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function avgRowHeight(config) {
  var contentHeight = config.contentHeight;
  var length = config.rows.length;

  return _fp2.default.assign(config, {
    avgRowHeight: length && _fp2.default.ceil(contentHeight / length)
  });
}

/**
 * Initializes the reduce call to reduceRowsIntoRowConfig. The step where we convert a list prop into lists is done here
 * @param  {array} list                     List or lists prop.
 * @param  {boolean} stackHeaders           Signifies that we should do the math to calculate stacking header offsets
 * @param  {Array}  [collapsedSections=[]]  The collapsedSection state from the scrollable class
 * @return {object}                         Reduces the list/lists into consumable state for the scrollable
 */
function buildRowConfig(list, stackHeaders) {
  var collapsedSections = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var offsetCount = constants.OFFSET_BUFFER - 1;

  if (list.length === 0) {
    return { contentHeight: 0, headers: null, partitions: [], rows: [] };
  }

  var isMultiList = Boolean(list[0] && list[0].headerComponent);

  var lists = isMultiList ? list : [{ rows: list }];

  var numRows = 0;
  var listIterator = lists.length;

  while (listIterator--) {
    numRows += 1 + lists[listIterator].rows.length;
  }

  if (!isMultiList) {
    numRows--;
  }

  var rows = new Array(numRows);
  var headers = [];
  var partitions = [];

  var controlIterator = numRows;
  var rowIterator = -1;
  var insertionIterator = 0;
  var adjustHeaderOffset = 0;
  var contentHeight = 0;
  var listItem = void 0;

  listIterator = 0;

  while (controlIterator--) {
    if (rowIterator === -1) {
      listItem = lists[listIterator];
      rowIterator++;

      if (listItem.headerComponent) {
        offsetCount++;
        offsetCount %= constants.OFFSET_BUFFER;

        if (offsetCount === 0) {
          partitions.push(contentHeight);
        }

        headers.push({
          adjustHeaderOffset: adjustHeaderOffset,
          index: insertionIterator,
          height: listItem.height,
          lockPosition: contentHeight - adjustHeaderOffset,
          realOffset: contentHeight
        });

        if (stackHeaders) {
          adjustHeaderOffset += listItem.height;
        }

        if (collapsedSections[listIterator] === undefined) {
          collapsedSections[listIterator] = Boolean(listItem.initCollapsed);
        }

        contentHeight += listItem.height;
        rows[insertionIterator++] = {
          className: listItem.headerClassName,
          contentComponent: listItem.headerComponent,
          height: listItem.height,
          isHeader: true,
          key: listItem.headerKey,
          props: listItem.headerProps
        };
      }

      if (isMultiList) {
        if (!listItem.rows.length) {
          listIterator++;
          rowIterator = -1;
        }
        continue;
      }
    }

    if (collapsedSections[listIterator]) {
      controlIterator -= listItem.rows.length - 1;
      continue;
    }

    var row = listItem.rows[rowIterator++];

    offsetCount++;
    offsetCount %= constants.OFFSET_BUFFER;

    if (offsetCount === 0) {
      partitions.push(contentHeight);
    }

    contentHeight += row.height;

    rows[insertionIterator++] = row;

    if (listItem.rows.length === rowIterator) {
      listIterator++;
      rowIterator = -1;
    }
  }

  return avgRowHeight({
    adjustHeaderOffset: adjustHeaderOffset,
    collapsedSections: collapsedSections,
    contentHeight: contentHeight,
    headers: headers,
    stackHeaders: stackHeaders,
    offsetCount: offsetCount,
    partitions: partitions,
    rows: rows
  });
}

var triggerAnimationFrameCreator = exports.triggerAnimationFrameCreator = function triggerAnimationFrameCreator(callback) {
  var needFrame = false;
  var renderingFrame = false;
  var oldParams = [];

  var triggerAnimationFrame = function triggerAnimationFrame() {
    for (var _len = arguments.length, params = Array(_len), _key = 0; _key < _len; _key++) {
      params[_key] = arguments[_key];
    }

    if (renderingFrame) {
      needFrame = true;
      oldParams = params;
    } else {
      renderingFrame = true;
      needFrame = false;
      requestAnimationFrame(function () {
        callback.apply(undefined, params);
        renderingFrame = false;
      });
      if (needFrame) {
        needFrame = false;
        triggerAnimationFrame.apply(undefined, _toConsumableArray(oldParams));
      }
    }
  };

  return triggerAnimationFrame;
};

/**
 * assign meaning to math
 * @param  {number} contentHeight the height of the content
 * @param  {number} offsetHeight  the offsetHeight from the vertical scrollbar
 * @return {number}               the difference between contentHeight and offsetHeight
 */
function getMaxHeight(contentHeight, offsetHeight) {
  return contentHeight - offsetHeight;
}

var getResizeValues = exports.getResizeValues = _fp2.default.cond([
// Dynamic column : LEFT ---------------------------------------------------------------------------------------------
[// resizing LEFT side
_fp2.default.isMatch({
  dynamicColumn: constants.columns.LEFT,
  side: constants.columns.LEFT
}), function (_ref) {
  var leftHandleWidth = _ref.leftHandleWidth;
  var rightExists = _ref.rightExists;
  var rightHandlePosition = _ref.rightHandlePosition;
  var rightHandleWidth = _ref.rightHandleWidth;
  var width = _ref.width;
  return {
    max: width - leftHandleWidth,
    mod: 1,
    min: rightExists ? rightHandlePosition + rightHandleWidth : 0
  };
}], [// resizing RIGHT side
_fp2.default.isMatch({
  dynamicColumn: constants.columns.LEFT,
  side: constants.columns.RIGHT
}), function (_ref2) {
  var leftExists = _ref2.leftExists;
  var leftHandlePosition = _ref2.leftHandlePosition;
  var rightHandleWidth = _ref2.rightHandleWidth;
  var width = _ref2.width;
  return {
    max: leftExists ? leftHandlePosition - rightHandleWidth : width - rightHandleWidth,
    mod: 1,
    min: 0
  };
}],
// dynamic column : RIGHT --------------------------------------------------------------------------------------------
[// resizing LEFT side
_fp2.default.isMatch({
  dynamicColumn: constants.columns.RIGHT,
  side: constants.columns.LEFT
}), function (_ref3) {
  var leftHandleWidth = _ref3.leftHandleWidth;
  var rightExists = _ref3.rightExists;
  var rightHandlePosition = _ref3.rightHandlePosition;
  var width = _ref3.width;
  return {
    max: rightExists ? rightHandlePosition - leftHandleWidth : width - leftHandleWidth,
    mod: -1,
    min: 0
  };
}], [// resizing RIGHT side
_fp2.default.isMatch({
  dynamicColumn: constants.columns.RIGHT,
  side: constants.columns.RIGHT
}), function (_ref4) {
  var leftExists = _ref4.leftExists;
  var leftHandlePosition = _ref4.leftHandlePosition;
  var leftHandleWidth = _ref4.leftHandleWidth;
  var rightHandleWidth = _ref4.rightHandleWidth;
  var width = _ref4.width;
  return {
    max: width - rightHandleWidth,
    mod: -1,
    min: leftExists ? leftHandlePosition + leftHandleWidth : 0
  };
}],
// dynamic column : MIDDLE -------------------------------------------------------------------------------------------
[// resizing LEFT side
_fp2.default.isMatch({
  dynamicColumn: constants.columns.MIDDLE,
  side: constants.columns.LEFT
}), function (_ref5) {
  var leftHandleWidth = _ref5.leftHandleWidth;
  var rightExists = _ref5.rightExists;
  var rightHandlePosition = _ref5.rightHandlePosition;
  var rightHandleWidth = _ref5.rightHandleWidth;
  var width = _ref5.width;
  return {
    max: rightExists ? width - (rightHandlePosition + rightHandleWidth) - leftHandleWidth : width - leftHandleWidth,
    mod: -1,
    min: 0
  };
}], [// resizing RIGHT side
_fp2.default.isMatch({
  dynamicColumn: constants.columns.MIDDLE,
  side: constants.columns.RIGHT
}), function (_ref6) {
  var leftExists = _ref6.leftExists;
  var leftHandlePosition = _ref6.leftHandlePosition;
  var leftHandleWidth = _ref6.leftHandleWidth;
  var rightHandleWidth = _ref6.rightHandleWidth;
  var width = _ref6.width;
  return {
    max: leftExists ? width - (leftHandlePosition + leftHandleWidth) - rightHandleWidth : width - rightHandleWidth,
    mod: 1,
    min: 0
  };
}]]);

var getMiddleColumnWidths = function getMiddleColumnWidths(_ref7) {
  var leftHandlePosition = _ref7.leftHandlePosition;
  var rightHandlePosition = _ref7.rightHandlePosition;
  return {
    leftGutterWidth: leftHandlePosition,
    rightGutterWidth: rightHandlePosition
  };
};
var getGutterWidths = exports.getGutterWidths = _fp2.default.cond([[_fp2.default.isMatch({ dynamicColumn: constants.columns.LEFT }), function (_ref8) {
  var leftHandlePosition = _ref8.leftHandlePosition;
  var leftHandleWidth = _ref8.leftHandleWidth;
  var rightHandlePosition = _ref8.rightHandlePosition;
  var width = _ref8.width;
  return {
    leftGutterWidth: width - leftHandlePosition - leftHandleWidth,
    rightGutterWidth: rightHandlePosition
  };
}], [_fp2.default.isMatch({ dynamicColumn: constants.columns.RIGHT }), function (_ref9) {
  var leftHandlePosition = _ref9.leftHandlePosition;
  var rightHandlePosition = _ref9.rightHandlePosition;
  var rightHandleWidth = _ref9.rightHandleWidth;
  var width = _ref9.width;
  return {
    leftGutterWidth: leftHandlePosition,
    rightGutterWidth: width - rightHandlePosition - rightHandleWidth
  };
}], [_fp2.default.isMatch({ dynamicColumn: constants.columns.MIDDLE }), getMiddleColumnWidths], [_fp2.default.stubTrue, getMiddleColumnWidths]]);

/**
 * calculates which partition is the top of our renderable content and clamps verticalTransform to within min/max values
 * @param  {number} preClampTransform vertical transform before we clamp it between min / max values
 * @param  {number} maxHeight         maximum height our vertical transform can reach
 * @param  {array} partitions         list of numbers which represent the starting vertical transformation of partitions
 *                                    by index (partitions[5] is the starting vertical transform of the 6th partition)
 * @return {object}                   topPartionIndex and the clamped verticalTransform
 */
function getVerticalScrollValues(preClampTransform, maxHeight, partitions) {
  var verticalTransform = _fp2.default.clamp(0, maxHeight, preClampTransform);
  var topPartitionIndex = _fp2.default.findIndex(function (partitionStartingIndex) {
    return partitionStartingIndex > verticalTransform;
  }, partitions) - 1;
  return { topPartitionIndex: topPartitionIndex, verticalTransform: verticalTransform };
}

/**
 * helper function that returns a very strict width object for gutters
 * @param  {Number} [width=0] the width we wish to express as a style object
 * @return {object}           a style object representing a the given width
 */
function getWidthStyle() {
  var width = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

  return _fp2.default.isNumber(width) && width > 0 ? {
    minWidth: width + 'px',
    width: width + 'px'
  } : undefined;
}

var returnWidthIfComponentExists = exports.returnWidthIfComponentExists = function returnWidthIfComponentExists(width, component) {
  return component ? width : 0;
};